//*********************************************************/
//*	Ordinal Terms alpha(n) and aalpha(n)                 */
//*********************************************************/
//
//   Libor Behounek, 1999
//
//  The program calculates the ordinal terms called alpha(n)
//    and alpha((n)) (the latter called aalpha(n) here)
//    for ordinals =< epsilon_0 - the n-th approximates
//    used for extending Ackermann hierarchy beyond omega.
//
//  Input: ordinal alpha
//  Output: list of alpha(n) and alpha((n)) up to n == 5.
//
//********************************************************/

#define __HEADER__
#include "ord_fnc.c"
#include <conio.h>

#define MAXN 5  // upper boundary for alpha(n) listing

const char heading [] = "\n\
****************************************\n\
 Alpha(n) Calculator 1.0\n\
****************************************\n\
Enter an ordinal expression using digits, omega, epsilon and operators +, -,\n\
*, ^. Example: omega^omega*2 + 5 + omega*3. Empty input breaks the loop.\n";

const char done [] = "---  quit  ---\n";

void PrintError ()
{
	switch (OrdError)
	{
		case ORD_ERR_SYNT:  printf ("Syntax Error."); break;
		case ORD_ERR_NAT:   printf ("Natural Number Overflow."); break;
		case ORD_ERR_DOM:   printf ("Domain Error."); break;
		case ORD_ERR_TOO:   printf ("Temporary or result too long."); break;
		case ORD_ERR_IMPL:  printf ("A not-yet-implemented feature."); break;
		default:            printf ("Undetermined Ordinal Error."); break;
	}
	OrdError = ORD_ERR_OK;
}

#define IS_BLANK(c) \
	((c) == ' '  ? 1 : \
	 (c) == '\t' ? 1 : 0)


void RemoveSpaces (char *s)
{
	for (char *ptr = s;
	     *ptr;
	     IS_BLANK (*ptr) ? *ptr++ : *s++ = *ptr++);
	*s = '\0';
}


void Loop ()
{
	char buffer [128];
	int finished = 0;
	Ord cnf, alpha, aalpha;
     int n;

	do
	{
		printf ("\n\n> ");
		buffer [0] = 126;  // max num of chars to be read
		cgets (buffer);    // string read begins at buffer+2
		printf ("\n");
		RemoveSpaces (buffer + 2);
		fflush (stdin);
		if (buffer [2] == '*' || buffer [1] == 0)
		{
			finished = 1;
			continue;
		}

		cnf.atoOrd (buffer + 2).Cantorize ();
		if (OrdError) PrintError ();
		else
		{
               printf ("\nCantor normal form:\n = ");
			cnf.PrintOn (stdout);
               printf ("\n");

               if (cnf > EPSILON)
               	printf ("\nThe number is greater than epsilon.");
               else if (!isLimit (cnf))
               	printf ("\nThe number is isolated.");
			else
               {
	               for (n = 1; n <= MAXN; ++n)
     	          {
          	          alpha = AlphaTerm (cnf, n);
               	     printf ("\nalpha (%d) = ", n);
                    	if (OrdError) PrintError ();
	                    else alpha.PrintOn (stdout);
     	          }
          	     printf("\n");
			}
     	}
	}
	while (! finished);
}


void main ()
{
	printf ("%s", heading);
	Loop ();
	printf ("%s", done);
}

