/********************************************************************/
/*  Ordinal Calculator 0.0                                          */
/********************************************************************/

#define __HEADER__
#include "ord_def.c"
#include <conio.h>


const char heading [] = "\n\
---  Ordinal Calculator  (version 1.2)  ---\n\
	Enter ? for help, * to quit.";

const char author []  = "Libor Behounek, 12-OCT-1995. Last change: 18-MAY-1996.";

const char help []   = "\
Enter an ordinal expression, composed of:\n\
  parentheses (), operators: +, -, *, ^ (taking customary precedence),\n\
  ordinal constants: omega, epsilon, omega1, and natural numbers.\n\
Evaluates: the Cantor normal form of the expression given.\n\
Example: (epsilon^2+1)^(omega1+3). Enter * to quit.";

const char done [] = "---  quit  ---\n";


void PrintError (const Ord & partial_result)
{
	switch (OrdError)
	{
		case ORD_ERR_SYNT:  printf ("Syntax Error."); break;
		case ORD_ERR_NAT:   printf ("Natural Number Overflow."); break;
		case ORD_ERR_DOM:   printf ("Domain Error."); break;
		case ORD_ERR_TOO:   printf ("Temporary or result too long."); break;
		case ORD_ERR_IMPL:  printf ("A not-yet-implemented feature."); break;
		default:            printf ("Undetermined Ordinal Error."); break;
	}
	OrdError = ORD_ERR_OK;
	printf ("  Partial result: ");
	partial_result.PrintOn (stdout);
}


#define IS_BLANK(c) \
	((c) == ' '  ? 1 : \
	 (c) == '\t' ? 1 : 0)


void RemoveSpaces (char *s)
{
	for (char *ptr = s;
	     *ptr;
	     IS_BLANK (*ptr) ? *ptr++ : *s++ = *ptr++);
	*s = '\0';
}


void Loop ()
{
	char buffer [128];
	int finished = 0;
	Ord result;

	do
	{
		printf ("\n> ");
		buffer [0] = 126;  // max num of chars to be read
		cgets (buffer);    // string read begins at buffer+2
		printf ("\n");
		RemoveSpaces (buffer + 2);
		fflush (stdin);
		if (buffer [2] == '*' || buffer [1] == 0)
		{
			finished = 1;
			continue;
		}
		if (buffer [2] == '?')
		{
			printf ("%s", help); 
			continue;
		}

		result.atoOrd (buffer + 2).Cantorize ();
		if (OrdError) PrintError (result);
		else
		{
			printf ("= ");
			result.PrintOn (stdout);
		}
	}
	while (! finished);
}


void main ()
{
	printf ("%s", heading);
	Loop ();
	printf ("%s", done);
}
