//*********************************************************/
//  Ordinal functions - addition to ORD_DEF
//*********************************************************/


#ifndef __ORDFNC
#ifndef __HEADER__
#define __HEADER__
#define __ORDFNC 1
#else
#define __ORDFNC 0
#endif  // __HEADER__

#include "ord_def.c"


/************************** interface *******************************/

Ord lastOrder (const Ord &alpha);
// order of the last term of Cantor normal form (cantorized)

int isLimit (const Ord &alpha);
// boolean: whether alpha is limit

Ord predecessor (const Ord &alpha);
// predecessor (cantorized)
// returns alpha if limit

Ord AlphaTerm (const Ord & alpha, int n);
// alpha(n) n-th "Ackermann" approximate of limit alpha
// returns alpha if alpha > EPSILON or alpha isolated

Ord AalphaTerm (const Ord & alpha, int n);
// alpha((n)):
//   alpha_0 = alpha, alpha_(i+1) = alpha(n)
//   iterate while alpha_i limit, take the last value.
// Stones computer for omega^(omega^omega)



//************************  implementation  *************************/

#if __ORDFNC


Ord lastOrder (const Ord &alpha)
{
     Ord result, *oorder;
	CantOrd calpha = CantOrd (alpha);
	CantOrd last = CantOrd (calpha, calpha.NumMemb ());
     CantOrd *order = last.GetOrder ();
     oorder = order -> ToOrd ();
     result = *oorder;
     delete (order);
     delete (oorder);
     return result;
}


int isLimit (const Ord &alpha)
{
	return (lastOrder (alpha) > 0);
}

Ord predecessor (const Ord &alpha)
// returns alpha if limit
{
	if (isLimit (alpha)) return alpha;

     CantOrd calpha = CantOrd (alpha);
     CantOrd allButLast = CantOrd (calpha, 1, calpha.NumMemb () - 1);
     CantOrd last = CantOrd (calpha, calpha.NumMemb ());
     CantOrd *lastOrder = last.GetOrder ();
     Ord *olastOrder = lastOrder -> ToOrd ();
     long lastMult = last.GetMult ();
     Ord *presult = allButLast.ToOrd ();

     Ord olast = OMEGA;
     olast ^= *olastOrder;
     olast *= (lastMult - 1);
     Ord result = *presult;
     result += olast;
     result.Cantorize ();

     delete (lastOrder);
     delete (olastOrder);
     delete (presult);
     return result;
}



Ord AlphaTerm (const Ord & alpha, int n)
{
	Ord result, temp;
     int i;

     if (!isLimit (alpha)) return alpha;
     if (alpha == EPSILON)
     {
     	result = OMEGA;
     	for (i = 1, temp = OMEGA; i < n; ++i)
          {
          	result = OMEGA;
          	result ^= temp;
               temp <<= result;
          }
     	return result.Cantorize ();
     }
     CantOrd calpha = CantOrd (alpha);
     CantOrd allButLast = CantOrd (calpha, 1, calpha.NumMemb () - 1);
     CantOrd last = CantOrd (calpha, calpha.NumMemb ());
     CantOrd *lastOrder = last.GetOrder ();
     long lastMult = last.GetMult ();
     Ord *olastOrder = lastOrder -> ToOrd ();
     Ord newButLast = OMEGA;
     newButLast ^= *olastOrder;
     newButLast *= (lastMult - 1);
     Ord newLast;
     if (!isLimit (*olastOrder))
     {
     	newLast = OMEGA;
     	newLast ^= predecessor (*olastOrder);
          newLast *= n;
     }
     else
     {
     	newLast = OMEGA;
          newLast ^= AlphaTerm (*olastOrder, n);
     }
     Ord *presult = allButLast.ToOrd ();
     result = *presult;
     result += newButLast;
     result += newLast;
     result.Cantorize ();

	delete (presult);
     delete (lastOrder);
     delete (olastOrder);
	return result;
}

Ord AalphaTerm (const Ord & alpha, int n)
// Stones computer for omega^(omega^omega)
{
     if (alpha >= EPSILON) return alpha;
     if (!isLimit (alpha)) return alpha;
     Ord ai, aii, result;
     aii <<= alpha;
     do
     {
     	ai = aii;
          result <<= ai;
     	aii = AlphaTerm (ai, n);
     }
     while (isLimit (ai));
	return result;
}

//*********************************************************/

#endif  // if __ORDFNC
#endif  // ifndef __ORDFNC
